#!/bin/bash

#Copyright (C) 2016 Adelmo De Santis
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#Versione 1.0 release date 22 maggio 2016

#Changelog:
#


#Temporizzazioni:
# minuti dal power loss / secondi / azione
#  90 5400 SHUTDOWN
#  30 1800 EMAIL


#Per debuggare l'applicazione si può usare il pin 16 come output
#scrivendo sopra i cambiamenti di stato con echo ed osservando quello
#che succede.

#Per abilitare il pin
# /bin/echo 201 > /sys/class/gpio/export
# /bin/echo out > /sys/class/gpio/gpio201/direction


#Dichiarazione delle variabili
LOG=/var/log/ac_powerfail
FLAG=/tmp/acpowerfail
SHUTDOWN=/tmp/shutdown
VALUE=/sys/class/gpio/gpio202/value
MAILFLAG=/tmp/acpowermailflag
SHMAILFLAG=/tmp/acpowershmailflag

TIMESTAMP=`/bin/date +%s`
DATETIME=`/bin/date +%Y-%m-%d_%H:%M:%S_%s`

echo "Debug log $LOG"
echo "Debug flag $FLAG"
echo "Debug shutdown $SHUTDOWN"
echo "Debug timestamp $TIMESTAMP"
echo "Debug datetime $DATETIME"
echo "Debug value $VALUE"

PF_FLAG=0
SH_FLAG=0
MLFLAG=0
SHMLFLAG=0
LAST_MAIL_FLAG=0
MAIL_TIME_FLAG=0
LAST_SHMAIL_FLAG=0
DELTATIME=0
DELTATIMEMAIL=0
DELTATIMESHMAIL=0

#sistemiamo il file di log, deve esistere
if [ ! -f /var/log/ac_powerfail ]; then
	echo "Debug Create log"
	touch /var/log/ac_powerfail
	chmod a+w /var/log/ac_powerfail
	else
	echo "Debug File di Log Esistente"
fi

#controllo dell'esistenza del valore di GPIO
if [ ! -f $VALUE ]; then
	echo "Debug Test existance VALUE"
	echo "Error Value File NOT-FOUND"
	echo "$DATETIME - File Value non trovato" >> $LOG
	exit 1
	else
	echo "GPIO OK"
fi
#Controlliamo se siamo già in powerfail, in caso affermativo
#calcoliamo il delta_T
if [ -f $FLAG ]; then
	echo "Debug Siamo in PowerFail"
	PF_FLAG=1;
	#calcolo del timestamp
	TIMESTAMP_LOSS=`cat $FLAG`
	DELTATIME=$(expr $TIMESTAMP - $TIMESTAMP_LOSS)
	echo "Debug stato power fail flag $PF_FLAG"
	echo "Debug deltatime $DELTATIME"
	else
	echo "Debug non siamo in powerfail"
fi

if [ -f $MAILFLAG ]; then
	echo "Debug almeno una mail inviata"
	MLFLAG=1;
	TIMESTAMP_MAIL=`cat $MAILFLAG`
	DELTATIMEMAIL=$(expr $TIMESTAMP - $TIMESTAMP_MAIL)
	else
	echo "Debug Non abbiamo inviato mail"
fi

if [ -f $SHMAILFLAG ]; then
	echo "Debug shutdown mail"
	SHMLFLAG=1;
	TIMESTAMP_SHMAIL=`cat $SHMAILFLAG`
	DELTATIMESHMAIL=$(expr $TIMESTAMP - $TIMESTAMP_SHMAIL)
	else
	echo "Debug flag shutdown non presente"
fi

#Controllo delle temporizzazioni

if [ $DELTATIME -ge 1800 ]; then
	echo "Debug (deltamail) - Sono passati più di 30 minuti dal powerfail"
	MAIL_TIME_FLAG=1
	else
	echo "Deltatime: $DELTATIME"
fi

if [ $DELTATIMEMAIL -ge 1800 ]; then
	echo "Debug (deltatimemail) - Sono passati più di 30 minuti dalla ultima mail"
	LAST_MAIL_FLAG=1
	else
	echo "Deltatimemail: $DELTATIMEMAIL"
fi

if [ $DELTATIMESHMAIL -ge 1800 ]; then
	echo "Debug (deltatimeSHmail)- Sono passati più di 30 minuti dalla ultima mail $DELTATIMESHMAIL"
	LAST_SHMAIL_FLAG=1
	else
	echo "LAST SHMAIL FLAG non settato sono meno di 1800 secondi dall'ultima mail"
fi

if [ $DELTATIME -ge 5400 ]; then
	echo "Debug (deltatime oltre 5400) - Sono passati piu di 90 minuti dal powerfail"
	SH_FLAG=1;
	else
	echo "SHUTDOWN FLAG non settato"
fi

#Controllo se siamo in SHUTDOWN
if [ -f $SHUTDOWN ]; then
	echo "Debug Siamo in shutdown"
	SH_FLAG=1;
	echo "Debug $SH_FLAG"
	else
	echo "SHUTDOWN FILE non settato"
fi


#Leggi il valore della variabile associata al PIN
value_read=`/bin/cat $VALUE`
echo "Debug Value $value_read"

#Casi: tensione presente con sistema regolare
if [ $value_read == 1 ] && [ $PF_FLAG == 0 ] && [ $SH_FLAG == 0 ] ; then
	#Situazione: siamo in powerok con tensione ok
	echo "Debug tutto ok"
	exit 0
fi

#Casi: tensione presente con sistema in powerfail
if [ $value_read == 1 ] && [ $PF_FLAG == 1 ] && [ $SH_FLAG==0 ]; then
	#Situazione: siamo in powerfail ma è tornata la tensione
	#entro i 90 minuti dal powerfail
	#Cancellazione del file ac_powerfail e del flag
	echo "Debug siamo in power_fail"
	echo "Debug tensione di rete presente"
	/bin/rm $FLAG
	PF_FLAG=0;
	echo "$DATETIME - Tensione Ripristinata" >> $LOG
	echo "$DATETIME Tensione Ripristinata" | mail -s "Casa_Power" <utente@maildomain>
	/bin/rm $MAILFLAG
        #inserire handler per telegram	
fi

#Casi: tensione presente con sistema in shutdown
if [ $value_read == 1 ] && [ $PF_FLAG == 1 ] && [ $SH_FLAG == 1 ]; then
	echo "Debug siamo in shutdown"
	echo "$DATETIME - Tensione Ripristinata - sistema in SHUTDOWN" >> $LOG	
	echo "$DATETIME Tensione Ripristinata - sistema in SHUTDOWN" | mail -s "Casa_Power" <utente@maildomain>
fi

#Casi: errore
if [ $value_read == 1 ] && [ $PF_FLAG == 0 ] && [ $SH_FLAG == 1 ]; then
	echo "Debug ERRORE 1"
	echo "$DATETIME - Errore 1 - Condizione Non prevista" >> $LOG
        echo "$DATETIME Errore - Condizione Non prevista" | mail -s "Casa_Power" <utente@maildomain>
fi
	


#Casi: tensione assente primo blackout
if [ $value_read == 0 ] && [ $PF_FLAG == 0 ] && [ $SH_FLAG == 0 ]; then
	echo "$TIMESTAMP" > $FLAG
	echo "Debug tensione assente, primo black out"
	echo "$DATETIME - BlackOut tensione primo evento" >> $LOG
        echo "$DATETIME Tensione di rete assente - Nuovo Blackout" | mail -s "Casa_Power" <utente@maildomain>
	echo "$TIMESTAMP" > $MAILFLAG	
fi

#Casi: tensione assente con blackout in corso
if [ $value_read == 0 ] && [ $PF_FLAG == 1 ] && [ $SH_FLAG == 0 ]; then
	echo "Debug tensione assente da $DELTATIME secondi"
        echo "$DATETIME - BlackOut tensione da $DELTATIME secondi" >> $LOG
	if [ $LAST_MAIL_FLAG == 1 ]; then
  	        echo "$DATETIME - BlackOut tensione da $DELTATIME secondi - invio mail" >> $LOG
		echo "$DATETIME Tensione di rete assente - $DELTATIME secondi" | mail -s "Casa_Power" <utente@maildomain>
		echo "$TIMESTAMP" > $MAILFLAG
	fi
		
fi

#Casi: tensione assente con blackout in corso e shutdown flag
if [ $value_read == 0 ] && [ $PF_FLAG == 1 ] && [ $SH_FLAG == 1 ]; then
	echo "Debug tensione assente da $DELTATIME secondi - SHUTDOWN"
        echo "$DATETIME - BlackOut tensione da $DELTATIME secondi - SHUTDOWN" >> $LOG
	echo "$TIMESTAMP" > $SHUTDOWN

	if [ ! -f $SHMAILFLAG ]; then
		echo "$TIMESTAMP" > $SHMAILFLAG
		echo "Debug - SHmail primo invio"
		echo "$DATETIME - BlackOut tensione da $DELTATIME secondi - SHUTDOWN Mail primo invio" >> $LOG
		echo "$DATETIME Tensione di rete assente 1 - $DELTATIME secondi SHUTDOWN" | mail -s "Casa_Power" <utente@maildomain>
	fi
	if [ $LAST_SHMAIL_FLAG == 1 ]; then
		echo "Debug - SHmail successivo invio"
		echo "$DATETIME - BlackOut tensione da $DELTATIME secondi - SHUTDOWN Mail successivo invio" >> $LOG
		echo "$DATETIME   Tensione di rete assente 2 - $DELTATIME secondi SHUTDOWN" | mail -s "Casa_Power" <utente@maildomain>
		echo "$TIMESTAMP" > $SHMAILFLAG
	fi
	#spegnimento del NAS
	/usr/bin/ssh username@ip_del_nas 'halt'
fi

#Casi: errore!
if [ $value_read == 0 ] && [ $PF_FLAG == 0 ] && [ $SH_FLAG == 1 ]; then
	echo "Debug ERRORE 2"
        echo "$DATETIME - Errore 2 - Condizione Non prevista" >> $LOG
        echo "$DATETIME Errore - Condizione Non prevista" | mail -s "Casa_Power" <utente@maildomain>
fi

